using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Windows.Forms;
using NeoTicker;
using System.Diagnostics;

namespace Luke.NTDotNet
{
    [System.Security.SuppressUnmanagedCodeSecurity]
    public class MultiIndicators : IIDLIndicator
    {
        IItselfObj Itself;
        INTLibObj NTLib;
        IDataObj Data1;
        IReportObj Report;
        IParamListObj Params;

        string sReport, _url;
        int IsDebug;

        XmlNodeList _LESignals = null;
        XmlNodeList _LXSignals = null;
        XmlNodeList _SESignals = null;
        XmlNodeList _SXSignals = null;
        WindowWrapper _wrapper = null;
        Luke.NTDotNet.Message _msgForm = null;

        double IIDLIndicator.IDLCallEx(NTIndicatorObjects NTIndicatorObjects)
        {
            this.Itself.UpdateByTick = true;
            Itself = NTIndicatorObjects.ItSelf();
            NTLib = NTIndicatorObjects.NTLib();
            Data1 = NTIndicatorObjects.Data1();
            Report = NTIndicatorObjects.Report();
            Params = NTIndicatorObjects.Params();

            if (Itself.FirstCall)
            {
                _wrapper = new WindowWrapper(GetProcessWindow());
                _msgForm = new Luke.NTDotNet.Message();
                _url = Params.get_Items("SignalPath").Str.ToString();
                IsDebug = Params.get_Items("Debug").Int;
                sReport = Itself.FuncName + "_" + Data1.Symbol;

                if (IsDebug == 1)
                {
                    Report.OpenNew(sReport);
                    Report.Clear(sReport);
                    Report.AddLine(sReport, "Initialized: " + sReport);
                    Report.AddLine(sReport, "---------------------");
                    Report.AddLine(sReport, "");
                }

                XmlDocument xmlDoc;
                XmlElement xmlElm;

                try
                {
                    xmlDoc = new XmlDocument();
                    if (_url.IndexOf(':') < 0)
                        _url = Application.StartupPath + "\\indicator\\" + _url;
                    xmlDoc.Load(_url);

                    xmlElm = xmlDoc.DocumentElement;
                    _LESignals = xmlElm.SelectNodes("longentry/formula");
                    _LXSignals = xmlElm.SelectNodes("longexit/formula");
                    _SESignals = xmlElm.SelectNodes("shortentry/formula");
                    _SXSignals = xmlElm.SelectNodes("shortexit/formula");

                }
                catch (System.Exception e)
                {
                    _msgForm.SetMessage("Critical Indicator Error","Unable to Load Signals from " + _url + "\nError: " + e.Message);
                    _msgForm.Show(_wrapper);
                    return 0;
                }
            }

            double dLongEntry = 0;
            if (_LESignals != null & _LESignals.Count > 0)
                dLongEntry = CalculateSignal("LE", _LESignals);
            double dLongExit = 0;
            if (_LXSignals != null & _LXSignals.Count > 0)
                dLongExit = CalculateSignal("LX", _LXSignals);
            double dShortEntry = 0;
            if (_SESignals != null & _SESignals.Count > 0)
                dShortEntry = CalculateSignal("SE", _SESignals);
            double dShortgExit = 0;
            if (_SXSignals != null & _SXSignals.Count > 0)
                dShortgExit = CalculateSignal("SX", _SXSignals);

            Itself.set_Plot(1, dLongEntry);
            Itself.set_Plot(2, dLongExit);
            Itself.set_Plot(3, 0 - dShortEntry);
            Itself.set_Plot(4, 0 - dShortgExit);
            Itself.set_Plot(5, 0);
            return 0;
        }

        public double CalculateSignal(string _sigName, XmlNodeList _nodeList)
        {
            double dTemp = 0;
            for (int i = 1; i <= _nodeList.Count; i++)
            {
                INestedIndicatorObj iTempObj;
                iTempObj = Itself.UpdateIndicator(_sigName + i.ToString());
                if (iTempObj == null)
                {
                    iTempObj = Itself.MakeIndicator(_sigName + i.ToString(), "fml", new string[] { "1" }, new string[] { ((XmlCDataSection)_nodeList[i - 1].FirstChild).Value });
                    if(IsDebug == 1)
                        Report.AddLine(sReport, "Created New Indicator");
                }
                dTemp += iTempObj.get_Value(0);
            }
            return dTemp;
        }
        #region NeoTicker IWin32Window retrieval
        protected IntPtr GetProcessWindow()
        {
            Process[] procs = Process.GetProcessesByName("NeoTicker");
            if (procs.Length != 0)
            {
                IntPtr hwnd = procs[0].MainWindowHandle;
                return hwnd;
            }
            else
                return IntPtr.Zero;
        }
    }
    public class WindowWrapper : System.Windows.Forms.IWin32Window
    {
        public WindowWrapper(IntPtr handle)
        {
            _hwnd = handle;
        }

        public IntPtr Handle
        {
            get { return _hwnd; }
        }
        private IntPtr _hwnd;
    }
        #endregion
    
}
